@@ -8,7 +8,9 @@
  * Extensive changes, 1999, Neil Brown <neilb@cse.unsw.edu.au>
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <string.h>
@@ -91,13 +93,22 @@ main(int argc, char **argv)
 		fprintf(stderr, "exportfs: -r and -u are incompatible.\n");
 		return 1;
 	}
-	if (optind == argc && ! f_all) {
-		xtab_export_read();
-		dump(f_verbose);
-		return 0;
-	}
-
 	new_cache = check_new_cache();
+	if (optind == argc && ! f_all) {
+		if (force_flush) {
+			if (new_cache)
+				cache_flush(1);
+			else {
+				fprintf(stderr, "exportfs: -f: only available with new cache controls: mount /proc/fs/nfsd first\n");
+				exit(1);
+			}
+			return 0;
+		} else {
+			xtab_export_read();
+			dump(f_verbose);
+			return 0;
+		}
+	}
 
 	if (f_export && ! f_ignore)
 		export_read(_PATH_EXPORTS);
@@ -121,7 +132,8 @@ main(int argc, char **argv)
 		if (!f_export)
 			for (i = optind ; i < argc ; i++)
 				unexportfs(argv[i], f_verbose);
-		rmtab_read();
+		if (!new_cache)
+			rmtab_read();
 	}
 	if (!new_cache) {
 		xtab_mount_read();
@@ -136,10 +148,43 @@ main(int argc, char **argv)
 	return export_errno;
 }
 
+static void
+exports_update_one(nfs_export *exp, int verbose)
+{
+		/* check mountpoint option */
+	if (exp->m_mayexport &&
+	    exp->m_export.e_mountpoint &&
+	    !is_mountpoint(exp->m_export.e_mountpoint[0]?
+			   exp->m_export.e_mountpoint:
+			   exp->m_export.e_path)) {
+		printf("%s not exported as %s not a mountpoint.\n",
+		       exp->m_export.e_path, exp->m_export.e_mountpoint);
+		exp->m_mayexport = 0;
+	}
+	if (exp->m_mayexport && ((exp->m_exported<1) || exp->m_changed)) {
+		if (verbose)
+			printf("%sexporting %s:%s to kernel\n",
+			       exp->m_exported ?"re":"",
+			       exp->m_client->m_hostname,
+			       exp->m_export.e_path);
+		if (!export_export(exp))
+			error(exp, errno);
+	}
+	if (exp->m_exported && ! exp->m_mayexport) {
+		if (verbose)
+			printf("unexporting %s:%s from kernel\n",
+			       exp->m_client->m_hostname,
+			       exp->m_export.e_path);
+		if (!export_unexport(exp))
+			error(exp, errno);
+	}
+}
+
+
 /* we synchronise intention with reality.
  * entries with m_mayexport get exported
  * entries with m_exported but not m_mayexport get unexported
- * looking at m_client->m_type == MCL_FQDN only
+ * looking at m_client->m_type == MCL_FQDN and m_client->m_type == MCL_GSS only
  */
 static void
 exports_update(int verbose)
@@ -147,33 +192,10 @@ exports_update(int verbose)
 	nfs_export 	*exp;
 
 	for (exp = exportlist[MCL_FQDN]; exp; exp=exp->m_next) {
-		/* check mountpoint option */
-		if (exp->m_mayexport && 
-		    exp->m_export.e_mountpoint &&
-		    !is_mountpoint(exp->m_export.e_mountpoint[0]?
-				   exp->m_export.e_mountpoint:
-				   exp->m_export.e_path)) {
-			printf("%s not exported as %s not a mountpoint.\n",
-			       exp->m_export.e_path, exp->m_export.e_mountpoint);
-			exp->m_mayexport = 0;
-		}
-		if (exp->m_mayexport && ((exp->m_exported<1) || exp->m_changed)) {
-			if (verbose)
-				printf("%sexporting %s:%s to kernel\n",
-				       exp->m_exported ?"re":"",
-				       exp->m_client->m_hostname,
-				       exp->m_export.e_path);
-			if (!export_export(exp))
-				error(exp, errno);
-		}
-		if (exp->m_exported && ! exp->m_mayexport) {
-			if (verbose)
-				printf("unexporting %s:%s from kernel\n",
-				       exp->m_client->m_hostname,
-				       exp->m_export.e_path);
-			if (!export_unexport(exp))
-				error(exp, errno);
-		}
+		exports_update_one(exp, verbose);
+	}
+	for (exp = exportlist[MCL_GSS]; exp; exp=exp->m_next) {
+		exports_update_one(exp, verbose);
 	}
 }
 			
@@ -364,7 +386,7 @@ dump(int verbose)
 				c = dumpopt(c, "wdelay");
 			if (ep->e_flags & NFSEXP_NOHIDE)
 				c = dumpopt(c, "nohide");
-			if (ep->e_flags & NFSEXP_CROSSMNT)
+			if (ep->e_flags & NFSEXP_CROSSMOUNT)
 				c = dumpopt(c, "crossmnt");
 			if (ep->e_flags & NFSEXP_INSECURE_PORT)
 				c = dumpopt(c, "insecure");
@@ -378,6 +400,8 @@ dump(int verbose)
 				c = dumpopt(c, "no_subtree_check");
 			if (ep->e_flags & NFSEXP_NOAUTHNLM)
 				c = dumpopt(c, "insecure_locks");
+			if (ep->e_flags & NFSEXP_NOACL)
+				c = dumpopt(c, "no_acl");
 			if (ep->e_flags & NFSEXP_FSID)
 				c = dumpopt(c, "fsid=%d", ep->e_fsid);
 			if (ep->e_mountpoint)
