@@ -6,7 +6,9 @@
  * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <sys/stat.h>
 #include <netinet/in.h>
@@ -30,6 +32,8 @@ enum auth_error
 
 static void		auth_fixpath(char *path);
 static char	*export_file = NULL;
+static nfs_export my_exp;
+static nfs_client my_client;
 
 extern int new_cache;
 
@@ -55,6 +59,7 @@ auth_reload()
 	last_modified = stb.st_mtime;
 
 	export_freeall();
+	memset(&my_client, 0, sizeof(my_client));
 	// export_read(export_file);
 	xtab_export_read();
 
@@ -69,8 +74,6 @@ auth_authenticate_internal(char *what, struct sockaddr_in *caller,
 	nfs_export		*exp;
 
 	if (new_cache) {
-		static nfs_export my_exp;
-		static nfs_client my_client;
 		int i;
 		/* return static nfs_export with details filled in */
 		if (my_client.m_naddr != 1 ||
@@ -80,6 +83,7 @@ auth_authenticate_internal(char *what, struct sockaddr_in *caller,
 			my_client.m_naddr = 0;
 			my_client.m_addrlist[0] = caller->sin_addr;
 			n = client_compose(caller->sin_addr);
+			*error = unknown_host;
 			if (!n)
 				return NULL;
 			strcpy(my_client.m_hostname, *n?n:"DEFAULT");
